from sdk32 import *

point = POINT()
ps = PAINTSTRUCT()
rect = RECT()
tm = TEXTMETRIC()

focus_num = 0
color = [0, 0, 0]
wnd_rect = None
wnd_scroll = []
wnd_label = []
wnd_value = []
old_wnd_procs = []
brush = []
color_label = ('Red', 'Green', 'Blue')

def scroll_proc(wnd, msg, wparam, lparam):
    global focus_num
    n = GetWindowLong(wnd, GWL_ID)
    if msg == WM_KEYDOWN:
        if wparam == VK_TAB:
            if GetKeyState(VK_SHIFT) < 0:
                n = (n + 2) % 3
            else:
                n = (n + 1) % 3
            SetFocus(wnd_scroll[n])
    elif wparam == WM_SETFOCUS:
        focus_num = n
    return CallWindowProc(old_wnd_procs[n], wnd, msg, wparam, lparam)

def wnd_proc(wnd, msg, wparam, lparam):
    if msg == WM_CREATE:
        brush.append(CreateSolidBrush(RGB(255, 0, 0)))
        brush.append(CreateSolidBrush(RGB(0, 255, 0)))
        brush.append(CreateSolidBrush(RGB(0, 0, 255)))
        return 0

    elif msg == WM_SIZE:
        x_client = LOWORD(lparam)
        y_client = HIWORD(lparam)

        dc = GetDC(wnd)
        GetTextMetrics(dc, tm)
        y_char = tm.height
        ReleaseDC(wnd, dc)

        MoveWindow(wnd_rect, 0, 0, x_client / 2, y_client, TRUE)

        for n in range(3):
            MoveWindow(wnd_scroll[n],
                       (2 * n + 1) * x_client / 14, 2 * y_char,
                       x_client / 14, y_client - 4 * y_char, TRUE)
            MoveWindow(wnd_label[n],
                       (4 * n + 1) * x_client / 28, y_char / 2,
                       x_client / 7, y_char, TRUE)
            MoveWindow(wnd_value[n],
                       (4 * n + 1) * x_client / 28, y_client - 3 * y_char / 2,
                       x_client / 7, y_char, TRUE)
        SetFocus(wnd)
        return 0

    elif msg == WM_SETFOCUS:
        SetFocus(wnd_scroll[focus_num])
        return 0

    elif msg == WM_VSCROLL:
        n = GetWindowLong(lparam, GWL_ID)
        if wparam == SB_PAGEDOWN:
            color[n] = min(255, color[n] + 16)
        elif wparam == SB_LINEDOWN:
            color[n] = min(255, color[n] + 1)
        elif wparam == SB_PAGEUP:
            color[n] = max(0, color[n] - 16)
        elif wparam == SB_LINEUP:
            color[n] = max(0, color[n] - 1)
        elif wparam == SB_TOP:
            color[n] = 0
        elif wparam == SB_BOTTOM:
            color[n] = 255
        elif LOWORD(wparam) in (SB_THUMBPOSITION, SB_THUMBTRACK):
            color[n] = HIWORD(wparam)
        SetScrollPos(wnd_scroll[n], SB_CTL, color[n], TRUE)
        SetWindowText(wnd_value[n], str(color[n]))
        DeleteObject(GetClassLong(wnd, GCL_HBRBACKGROUND))
        SetClassLong(wnd, GCL_HBRBACKGROUND,
                     CreateSolidBrush(RGB(color[0], color[1], color[2])))
        InvalidateRect(wnd, None, TRUE)
        return 0

    elif msg == WM_CTLCOLORSCROLLBAR:
        SetBkColor(wparam, GetSysColor(COLOR_CAPTIONTEXT))
        SetTextColor(wparam, GetSysColor(COLOR_WINDOWFRAME))
        n = GetWindowLong(LOWORD(lparam), GWL_ID)
        point.x = point.y = 0
        ClientToScreen(wnd, point)
        UnrealizeObject(brush[n])
        SetBrushOrgEx(wparam, point.x, point.y, None)
        return brush[n]

    elif msg == WM_DESTROY:
        DeleteObject(GetClassLong(wnd, GCL_HBRBACKGROUND))
        for n in range(3):
            DeleteObject(brush[n])
        PostQuitMessage(0)
        return 0

    return DefWindowProc(wnd, msg, wparam, lparam)

def winmain(inst, prev_inst, cmd_line, cmd_show):
    global wnd_rect
    app_name = 'Colors1'

    wc = WNDCLASSEX()
    wc.style = CS_HREDRAW | CS_VREDRAW
    wc.wnd_proc = wnd_proc
    wc.inst = inst
    wc.cursor = LoadCursor(0, IDC_ARROW)
    wc.background = GetStockObject(WHITE_BRUSH)
    wc.name = app_name
    RegisterClassEx(wc)

    wnd = CreateWindow(app_name, 'Color Scroll',
                       WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, inst, 0)

    wnd_rect = CreateWindow('static', None,
                            WS_CHILD | WS_VISIBLE | SS_WHITERECT,
                            0, 0, 0, 0,
                            wnd, 9, inst, None)
    for n in range(3):
        scroll = CreateWindow('scrollbar', None,
                              WS_CHILD | WS_VISIBLE | WS_TABSTOP | SBS_VERT,
                              0, 0, 0, 0,
                              wnd, n, inst, None)
        wnd_scroll.append(scroll)
        label =  CreateWindow('static', color_label[n],
                              WS_CHILD | WS_VISIBLE | SS_CENTER,
                              0, 0, 0, 0,
                              wnd, n + 3, inst, None)
        wnd_label.append(label)
        value =  CreateWindow('static', '0',
                              WS_CHILD | WS_VISIBLE | SS_CENTER,
                              0, 0, 0, 0,
                              wnd, n + 6, inst, None)
        wnd_value.append(value)
        old_wnd_procs.append(GetWindowLong(scroll, GWL_WNDPROC))
        SetWindowLong(scroll, GWL_WNDPROC, scroll_proc)
        SetScrollRange(scroll, SB_CTL, 0, 255, FALSE)
        SetScrollPos(scroll, SB_CTL, 0, FALSE)

    ShowWindow(wnd, cmd_show)
    UpdateWindow(wnd)

    msg = MSG()
    while 1:
        if not GetMessage(msg, 0, 0, 0):
            break
        TranslateMessage(msg)
        DispatchMessage(msg)
    return msg.wparam

if __name__ == '__main__':
    winmain(GetModuleHandle(None), 0, '', SW_SHOWNORMAL)
